# extract features

from PIL import Image
from math import pi, sqrt, cos, sin
from PIL.ImageDraw import Draw 


def dtor (d):
    return (d/360.0)*2.0*pi

def cutout (im, x, y, r, angle, showVector=True, vectorColor=(255, 0, 0), vectorWidth=5):
    """ angle is degrees """
    # d.rectangle((x-r, y-r, x+r, y+r), fill=None, outline=(0, 0, 255))
    # d.ellipse((x-r, y-r, x+r, y+r), fill=(255, 255, 255))
    # Circumscribe a square around the circle
    rc = int(r * sqrt(2))
    #d.rectangle((x-rc, y-rc, x+rc, y+rc), fill=None, outline=(0, 255, 255))
    #d.ellipse((x-rc, y-rc, x+rc, y+rc), fill=None, outline=(0, 255, 255))

    maskwidth = int(rc*2)
    hmaskwidth = maskwidth / 2
    mask = Image.new("L", (maskwidth, maskwidth), color=0)
    md = Draw(mask)
    md.rectangle((hmaskwidth-r, hmaskwidth-r, hmaskwidth+r, hmaskwidth+r), fill=255, outline=None)

    # mask = mask.rotate(angle)
    mask = mask.rotate(angle, resample=Image.BICUBIC)

    # Using L mode has no alpha
    cutout = Image.new("RGBA", (maskwidth, maskwidth))
    src = im.crop((x-rc, y-rc, x+rc, y+rc))
    cutout.paste(src, mask=mask)

    # cd.pieslice((hmaskwidth-r, hmaskwidth-r, hmaskwidth+r, hmaskwidth+r), -45, 0, fill=(255, 0, 0))
    # calculate the direction vector endpoint
    if showVector:
        cd = Draw(cutout)
        dvx = hmaskwidth + (r * cos(dtor(angle)))
        dvy = hmaskwidth - (r * sin(dtor(angle)))
        # cd.line((hmaskwidth, hmaskwidth, hmaskwidth, 0), fill=(255, 0, 0), width=5)
        cd.line((hmaskwidth, hmaskwidth, dvx, dvy), fill=vectorColor, width=vectorWidth)
    return cutout

if __name__ == "__main__":
    import sys

    image_path = sys.argv[1]
    im = Image.open(image_path)
    #d = Draw(im)

    x, y = 217, 150
    r = 45

    for angle in range(0, 360, 10):
        c = cutout(im, x, y, r, angle, vectorColor=(255,255,0))
        c.save("cutout{0:03d}.png".format(angle))

